@@ -35,12 +35,16 @@ GSList *device_scan(void)
 	int i;
 
 	if (opt_drv) {
+		/* Caller specified driver. Use it. Only this one. */
 		if (!parse_driver(opt_drv, &driver, &drvopts))
 			return NULL;
 		devices = sr_driver_scan(driver, drvopts);
 		g_slist_free_full(drvopts, (GDestroyNotify)free_drvopts);
+	} else if (opt_dont_scan) {
+		/* No -d choice, and -D "don't scan" requested. Do nothing. */
+		devices = NULL;
 	} else {
-		/* No driver specified, let them all scan on their own. */
+		/* No driver specified. Scan all available drivers. */
 		devices = NULL;
 		drivers = sr_driver_list(sr_ctx);
 		for (i = 0; drivers[i]; i++) {
@@ -59,16 +63,37 @@ GSList *device_scan(void)
 	return devices;
 }
 
-struct sr_channel_group *select_channel_group(struct sr_dev_inst *sdi)
+/**
+ * Lookup a channel group from its name.
+ *
+ * Uses the caller specified channel group name, or a previously stored
+ * option value as a fallback. Returns a reference to the channel group
+ * when the lookup succeeded, or #NULL after lookup failure, as well as
+ * #NULL for the global channel group (the device).
+ *
+ * Accepts either #NULL pointer, or an empty string, or the "global"
+ * literal to address the global channel group (the device). Emits an
+ * error message when the lookup failed while a name was specified.
+ *
+ * @param[in] sdi Device instance.
+ * @param[in] cg_name Caller provided channel group name.
+ *
+ * @returns The channel group, or #NULL for failed lookup.
+ */
+struct sr_channel_group *lookup_channel_group(struct sr_dev_inst *sdi,
+	const char *cg_name)
 {
 	struct sr_channel_group *cg;
 	GSList *l, *channel_groups;
 
-	if (!opt_channel_group)
+	if (!cg_name)
+		cg_name = opt_channel_group;
+	if (cg_name && g_ascii_strcasecmp(cg_name, "global") == 0)
+		cg_name = NULL;
+	if (!cg_name || !*cg_name)
 		return NULL;
 
 	channel_groups = sr_dev_inst_channel_groups_get(sdi);
-
 	if (!channel_groups) {
 		g_critical("This device does not have any channel groups.");
 		return NULL;
@@ -76,11 +101,11 @@ struct sr_channel_group *select_channel_group(struct sr_dev_inst *sdi)
 
 	for (l = channel_groups; l; l = l->next) {
 		cg = l->data;
-		if (!g_ascii_strcasecmp(opt_channel_group, cg->name)) {
-			return cg;
-		}
+		if (g_ascii_strcasecmp(cg_name, cg->name) != 0)
+			continue;
+		return cg;
 	}
-	g_critical("Invalid channel group '%s'", opt_channel_group);
+	g_critical("Invalid channel group '%s'", cg_name);
 
 	return NULL;
 }
